//
// Created by Laky64 on 08/08/2023.
//

#pragma once

#include <wrtc/exceptions.hpp>
#include <wrtc/enums.hpp>
#include <wrtc/models/frame.hpp>
#include <wrtc/interfaces/media/rtc_audio_source.hpp>
#include <wrtc/interfaces/media/rtc_video_source.hpp>
#include <wrtc/utils/binary.hpp>
#include <wrtc/utils/bignum.hpp>
#include <wrtc/utils/random.hpp>
#include <wrtc/utils/g_zip.hpp>
#include <wrtc/models/ice_candidate.hpp>