//
// Created by Laky64 on 29/08/2023.
//

#pragma once

namespace ntgcalls {

    struct MediaState {
        bool muted;
        bool videoPaused;
        bool videoStopped;
        bool presentationPaused;
    };

} // ntgcalls
